@@ -43,6 +43,11 @@ class TestAuthenticateMessage(unittest.TestCase):
     def setUp(self):
         self.message = read_test_data("test.message")
         self.message2 = read_test_data("test.message_signed")
+        self.message3 = read_test_data("test.message_multi")
+        self.message4 = read_test_data("test.message_multi2")
+        self.message5 = read_test_data("test.message_sub")
+        self.message6 = read_test_data("test.message_np1")
+        self.message7 = read_test_data("test.message_np2")
         self.key = read_test_data("test.private")
 
     def dnsfunc(self, domain):
@@ -57,7 +62,15 @@ MEeWUwg8Istjqz8BZeTWbf41fbNhte7Y+YqZOwq1Sd0DbvYAD9NOZK9vlfuac0598H\
 Y+vtSBczUiKERHv1yRbcaQtZFh5wtiRrN04BLUTD21MycBX5jYchHjPY/wIDAQAB""",
           "_dmarc.example.com": """v=DMARC1\; p=reject\;\
  rua=mailto:dmarc.reports@valimail.com,mailto:dmarc_agg@vali.email\;\
- ruf=mailto:dmarc.reports@valimail.com,mailto:dmarc_c0cb7153_afrf@vali.email"""
+ ruf=mailto:dmarc.reports@valimail.com,mailto:dmarc_c0cb7153_afrf@vali.email""",
+          "_dmarc.example.org": """v=DMARC1\; p=none""",
+          "_dmarc.example.net": """v=DMARC1\; p=none\; sp=reject""",
+          "_dmarc.sub.example.net": "",
+          "_dmarc.example.biz": """v=DMARC1\; p=none\; sp=quarantine\; np=reject""",
+          "_dmarc.sub.example.biz": "",
+          "_dmarc.sub2.example.biz": "",
+          "sub.example.biz": None,
+          "sub2.example.biz": "host.example.biz"
         }
         try:
             if isinstance(domain, bytes):
@@ -74,7 +87,32 @@ Y+vtSBczUiKERHv1yRbcaQtZFh5wtiRrN04BLUTD21MycBX5jYchHjPY/wIDAQAB""",
 
     def test_authenticate_dmarc(self):
         res = authenticate_message(self.message2, "example.com", spf=False, dnsfunc=self.dnsfunc)
-        self.assertEqual(res, "Authentication-Results: example.com; dkim=pass header.d=example.com header.i=@example.com; dmarc=pass (Used From Domain Record) header.from=example.com")
+        self.assertEqual(res, "Authentication-Results: example.com; dkim=pass header.d=example.com header.i=@example.com; dmarc=pass (Used From Domain Record) header.from=example.com policy.dmarc=reject")
+
+    def test_authenticate_dmarc_mult_from(self):
+        self.maxDiff = None
+        res = authenticate_message(self.message3, "example.com", prev='Authentication-Results: example.com; dkim=fail header.d=example.com header.i=@example.com; dkim=pass header.d=example.org header.i=@example.org', spf=False, dkim=False, dnsfunc=self.dnsfunc)
+        self.assertEqual(res, "Authentication-Results: example.com; dkim=fail header.d=example.com header.i=@example.com; dkim=pass header.d=example.org header.i=@example.org; dmarc=fail (Used From Domain Record) header.from=example.com policy.dmarc=reject")
+
+    def test_authenticate_dmarc_mult_from2(self):
+        self.maxDiff = None
+        res = authenticate_message(self.message4, "example.com", prev='Authentication-Results: example.com; dkim=fail header.d=example.com header.i=@example.com; dkim=pass header.d=example.org header.i=@example.org', spf=False, dkim=False, dnsfunc=self.dnsfunc)
+        self.assertEqual(res, "Authentication-Results: example.com; dkim=fail header.d=example.com header.i=@example.com; dkim=pass header.d=example.org header.i=@example.org; dmarc=fail (Used From Domain Record) header.from=example.com policy.dmarc=reject")
+
+    def test_authenticate_dmarc_sub(self):
+        self.maxDiff = None
+        res = authenticate_message(self.message5, "example.com", prev='Authentication-Results: example.com; dkim=fail header.d=sub.example.net header.i=@sub.example.net', spf=False, dkim=False, dnsfunc=self.dnsfunc)
+        self.assertEqual(res, "Authentication-Results: example.com; dkim=fail header.d=sub.example.net header.i=@sub.example.net; dmarc=fail (Used Org Domain Record) header.from=sub.example.net policy.dmarc=reject")
+
+    def test_authenticate_dmarc_np1(self):
+        self.maxDiff = None
+        res = authenticate_message(self.message6, "example.com", prev='Authentication-Results: example.com; dkim=fail header.d=sub.example.biz header.i=@sub.example.biz', spf=False, dkim=False, dnsfunc=self.dnsfunc)
+        self.assertEqual(res, "Authentication-Results: example.com; dkim=fail header.d=sub.example.biz header.i=@sub.example.biz; dmarc=fail (Used Org Domain Record) header.from=sub.example.biz policy.dmarc=reject")
+
+    def test_authenticate_dmarc_np2(self):
+        self.maxDiff = None
+        res = authenticate_message(self.message7, "example.com", prev='Authentication-Results: example.com; dkim=fail header.d=sub2.example.biz header.i=@sub2.example.biz', spf=False, dkim=False, dnsfunc=self.dnsfunc)
+        self.assertEqual(res, "Authentication-Results: example.com; dkim=fail header.d=sub2.example.biz header.i=@sub2.example.biz; dmarc=fail (Used Org Domain Record) header.from=sub2.example.biz policy.dmarc=quarantine")
 
     def test_prev(self):
         prev = "Authentication-Results: example.com; spf=pass smtp.mailfrom=gmail.com"
